/*
 * FatDirOrderChanger: FATファイルシステムディレクトリエントリ並び順変更
 */

package org.dbc_works.app.sbfiles;

import java.io.File;
import java.util.Observer;

public final class FatEntryOrderChanger
{
	//
	// 内部定義型
	//

	// Result: 結果
	// 『Effective Java』「項目21 enum構文をクラスで置き換える」に従って実装
	public static final class Result
	{
		//
		// クラス属性
		//

		public static final Result SUCCESS	= new Result("Success.");

		public static final Result FAIL_CREATE_TEMP_DIR	= new Result("Fail: Create temp directory.");
		public static final Result FAIL_MOVE_ENTRY	= new Result("Fail: Move entry.");
		public static final Result FAIL_DELETE_TEMP_DIR	= new Result("Fail: Delete temp directory.");

		//
		// 属性
		//
		final String name;

		//
		// コンストラクタ
		//
		private Result(
			String strName)	// 名前
		{
			 name = strName;
		}
	}

	//
	// クラス属性
	//
	private static final boolean	DEBUG	= false;

	//
	// 属性
	//
	private File	target;
	private File[]	updateEntries;

	private CountReporter	reporter;

	//
	// コンストラクタ
	//
	public FatEntryOrderChanger(
		File fileTarget,			// 作業ディレクトリ
		File[] listUpdate,			// 更新エントリ
		Observer reportReceiver)	// レポート受信クラス 
	{
		if (DEBUG) {
			if (fileTarget == null) {
				throw new NullPointerException();
			}
			else if (fileTarget.exists() == false || fileTarget.isDirectory() == false) {
				throw new IllegalArgumentException();
			}
			if (listUpdate == null) {
				throw new NullPointerException();
			}
			else if (listUpdate.length == 0) {
				throw new IllegalArgumentException();
			}
			
			for (int nIndex = 0; nIndex < listUpdate.length; nIndex++) {
				File fileUpdate = listUpdate[nIndex];
				if (fileUpdate.getParentFile().compareTo(fileTarget) != 0) { 
					throw new IllegalArgumentException();
				}
			}
		}

		target = fileTarget;
		updateEntries = listUpdate;

		if (reportReceiver != null) {
			reporter = new CountReporter(reportReceiver, updateEntries.length * 2);
		}
	}
	
	//
	// 操作
	//

	// 実行する
	// 注記: 速度を重視するなら一時ディレクトリへの移動後元ディレクトリを削除して
	//	一時ディレクトリを元ディレクトリの名前に変更すればいい。しかしそうすると
	//	親ディレクトリが移動してしまうので現在のやりかたとしている。
	public Result execute()
	{
		if (DEBUG) {
			System.err.println("FatDirOrderChanger: Start.");
		}

		Result result = Result.SUCCESS;

		// 移動用の一時ディレクトリを作成する
		File fileTempDir = new File(target.getAbsolutePath() + ".tmp");
		while (fileTempDir.exists()) {
			fileTempDir = new File(fileTempDir.getAbsolutePath() + ".tmp");
		}

		if (DEBUG) {
			System.err.println("\tCreate temp directory.");
		}

		if (fileTempDir.mkdir()) {
			// 一時ディレクトリに移動する
			if (DEBUG) {
				System.err.println("\tMove to temporary:");
			}
			if (MoveEntry(updateEntries, fileTempDir) == false) {
				result = Result.FAIL_MOVE_ENTRY;
			}
		
			// 移動した順に戻す
			// ここまででエラーが起きていたら移動した分の復帰を試みることになる
			if (DEBUG) {
				System.err.println("\tMove to source:");
			}
	 		if (MoveEntry(fileTempDir.listFiles(), target) == false) {
				if (result == Result.SUCCESS) {
					result = Result.FAIL_MOVE_ENTRY;
				}
	 		}

			// 一時ディレクトリを削除する
			// ここまででエラーが起きていたときは削除できないかもしれない
			if (DEBUG) {
				System.err.println("\tDelete temp directory.");
			}
			if (fileTempDir.delete() == false) {
				if (result == Result.SUCCESS) {
					result = Result.FAIL_DELETE_TEMP_DIR;
				}
			}
		}
		else {
			result = Result.FAIL_CREATE_TEMP_DIR;
		}

		if (DEBUG) {
			System.err.println("\tEnd.");
			if (result != Result.SUCCESS) {
				System.err.println("\tError; " + result.name);
			}
		}

		return result;
	}

	//
	// 実装
	//

	// エントリを移動する 
	private boolean MoveEntry(
		File[] aFile,	// エントリの集合
		File fileDst)	// 移動先ディレクトリ
	{
		boolean bSuccess = true;
		for (int nIndex = 0; nIndex < aFile.length; nIndex++) {
			File fileSrc = aFile[nIndex]; 
			File fileMove = new File(fileDst, fileSrc.getName()); 
			if (DEBUG) {
				System.err.println("\t\t" + fileMove.getAbsolutePath());
			}
			bSuccess = fileSrc.renameTo(fileMove);
			if (bSuccess == false) {
				break;
			}

			if (reporter != null) {
				reporter.reportCountUp();
			}
		}

		return bSuccess;
	}
}
