/*
 * GUnzipCommand: GZIP伸長コマンド
 */

package org.dbc_works.app.sbfiles.cmd;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.GZIPInputStream;

import org.dbc_works.lib.io.FileUtilities;
import org.dbc_works.lib.util.zip.ZipUtilities;
import org.dbc_works.lib.yaml.YamlSerializer;

import org.dbc_works.app.sbfiles.CountReporter;
import org.dbc_works.app.sbfiles.MainFrame;
import org.dbc_works.app.sbfiles.view.LabelManager;

public final class GUnzipCommand extends ProcessEachEntryCommand
{
	//
	// クラスメソッド(実装)
	//
	
	// GZIP伸長ファイルのファイル名を作成する
	private static File makeGUnzipedEntry(
		File sourceEntry)	// 伸長対象ファイル
	{
		if (sourceEntry == null) {
			throw new NullPointerException();
		}
		
		String strGUnzipedName = FileUtilities.separateNameAndExt(sourceEntry.getAbsolutePath())[0];
		String[] astrOriginalName = FileUtilities.separateNameAndExt(strGUnzipedName);

		int nIndex = 2;
		File gunzipEntry = new File(strGUnzipedName);
		while (gunzipEntry.exists()) {
			strGUnzipedName = FileUtilities.combineNameAndExt(astrOriginalName[0],
															Integer.toString(nIndex));
			if (0 < astrOriginalName[1].length()) {
				strGUnzipedName = FileUtilities.combineNameAndExt(strGUnzipedName,
																	astrOriginalName[1]);
			}
			gunzipEntry = new File(strGUnzipedName);
			nIndex++;
		}
		return gunzipEntry;
	}

	//
	// 属性
	//

	// 伸長対象ファイル	
	private final File[] targetEntries;
	
	// 伸長済みファイル
	private final File[] gunzipedEntries;
	
	//
	// コンストラクタ
	//
	public GUnzipCommand(
		File fileTarget,	// 対象ディレクトリ
		File[] entries,		// 伸長対象エントリ 
		MainFrame frame)	//
	{
		super(LabelManager.getCommandNameProperties().getProperty("Command_GUnzip"), fileTarget, frame);
	
		if (entries == null) {
			throw new NullPointerException();
		}
		else if (entries.length == 0) {
			throw new IllegalArgumentException();
		}

		targetEntries = entries;
		for (int nIndex = 0; nIndex < targetEntries.length; nIndex++) {
			if (targetEntries[nIndex].exists() == false) {
				throw new IllegalStateException();
			}
			if (targetEntries[nIndex].isDirectory()) {
				throw new IllegalArgumentException();
			}
			else if (ZipUtilities.getFileType(targetEntries[nIndex]) != ZipUtilities.FileType.GZIP) {
				throw new IllegalArgumentException();
			}
		}
		gunzipedEntries = new File[targetEntries.length];
	}

	//
	// 再定義メソッド(Command)
	//

	// 実行可能かを返す
	public boolean canExecute()
	{
		boolean canExecute = true;
		if (isExecuted() == false) {
			for (int nIndex = 0; nIndex < targetEntries.length; nIndex++) {
				if (targetEntries[nIndex].exists() == false
				|| targetEntries[nIndex].isDirectory()
				|| ZipUtilities.getFileType(targetEntries[nIndex]) != ZipUtilities.FileType.GZIP) {
					canExecute = false;
					break;
				}
			}
		}
		else {
			for (int nIndex = 0; nIndex < gunzipedEntries.length; nIndex++) {
				if (gunzipedEntries[nIndex] == null
				|| gunzipedEntries[nIndex].exists() == false
				|| gunzipedEntries[nIndex].isDirectory()) {
					canExecute = false;
					break;
				}
			}
		}
		
		return canExecute;
	}

	//
	// 再定義メソッド(ProcessEachEntryCommand)
	//

	// 処理エントリ数を返す
	protected int getProcessEntryCount()
	{
		return targetEntries.length;
	}
	
	// 処理名を返す
	protected String getOperationName()
	{
		return (this.isExecuted() == false ? "GZip file(s)" : "Cancel(delete) GZiped file(s)");
	}
	
	// 実行する	
	protected void process(
		YamlSerializer logSerializer,	// ログ書き込み
		CountReporter reporter)			// レポーター
		throws IOException
	{
		if (logSerializer == null) {
			throw new NullPointerException();
		}
		if (reporter == null) {
			throw new NullPointerException();
		}

		for (int nIndex = 0; nIndex < targetEntries.length; nIndex++) {
			// GZIP伸長ファイル名を作成する
			// この時点でファイル名をフィールドに保存する
			gunzipedEntries[nIndex] = makeGUnzipedEntry(targetEntries[nIndex]);

			InputStream  from = null;
			OutputStream to = null;
			try {
				from = new GZIPInputStream(new BufferedInputStream(new FileInputStream(targetEntries[nIndex])));
				to = new BufferedOutputStream(new FileOutputStream(gunzipedEntries[nIndex]));
				FileUtilities.duplicateStream(from, to);
			}
			finally {
				if (to != null) {
					to.close();
				}
				if (from != null) {
					from.close();
				}
			}
				
			System.err.println(targetEntries[nIndex].getName()
								+ " -> "
								+ gunzipedEntries[nIndex].getName());
			logSerializer.writeMapping(targetEntries[nIndex].getName(), gunzipedEntries[nIndex].getName());
			reporter.reportCountUp();
		}
	}
	
	// 処理をキャンセルする
	protected void cancelProcess(
		YamlSerializer logSerializer,	// ログ書き込み
		CountReporter reporter)			// レポーター
		throws IOException
	{
		if (logSerializer == null) {
			throw new NullPointerException();
		}

		// GZIP伸長したファイルを削除する
		for (int nIndex = 0; nIndex < gunzipedEntries.length; nIndex++) {
			// 伸長失敗時も処理するため参照をチェックする
			if (gunzipedEntries[nIndex] != null) {
				if (gunzipedEntries[nIndex].exists()) {
					gunzipedEntries[nIndex].delete();
				}
				
				System.err.println("Delete: " + gunzipedEntries[nIndex].getName());
				logSerializer.writeSequence(gunzipedEntries[nIndex].getName());
				if (reporter != null) {
					reporter.reportCountUp();
				}
				
				gunzipedEntries[nIndex] = null;
			}
		}
	}
}
