/*
 * GZipCommand: GZIP圧縮コマンド
 */

package org.dbc_works.app.sbfiles.cmd;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;

import org.dbc_works.lib.io.FileUtilities;
import org.dbc_works.lib.util.zip.ZipUtilities;
import org.dbc_works.lib.yaml.YamlSerializer;

import org.dbc_works.app.sbfiles.CountReporter;
import org.dbc_works.app.sbfiles.MainFrame;
import org.dbc_works.app.sbfiles.view.LabelManager;

public final class GZipCommand extends ProcessEachEntryCommand
{
	//
	// クラスメソッド(実装)
	//
	
	// GZIP圧縮ファイルのファイル名を作成する
	private static File makeGZipEntry(
		File sourceEntry)	// 圧縮対象ファイル
	{
		if (sourceEntry == null) {
			throw new NullPointerException();
		}
		
		String strGZipName = FileUtilities.combineNameAndExt(sourceEntry.getAbsolutePath(), ZipUtilities.FileType.GZIP.extension);
		int nIndex = 2;
		File gzipEntry = new File(strGZipName);
		while (gzipEntry.exists()) {
			String[] pathInfo = FileUtilities.separateNameAndExt(sourceEntry.getAbsolutePath());
			String newFileName = FileUtilities.combineNameAndExt(FileUtilities.combineNameAndExt(pathInfo[0], Integer.toString(nIndex)),
																pathInfo[1]);
			strGZipName = FileUtilities.combineNameAndExt(newFileName, ZipUtilities.FileType.GZIP.extension);
			gzipEntry = new File(strGZipName);
			nIndex++;
		}
		return gzipEntry;
	}

	//
	// 属性
	//

	// 圧縮対象ファイル	
	private final File[] targetEntries;
	
	// 圧縮済みファイル
	private final File[] gzipedEntries;
	
	//
	// コンストラクタ
	//
	public GZipCommand(
		File fileTarget,	// 対象ディレクトリ
		File[] entries,		// 圧縮対象エントリ 
		MainFrame frame)	//
	{
		super(LabelManager.getCommandNameProperties().getProperty("Command_GZip"), fileTarget, frame);
	
		if (entries == null) {
			throw new NullPointerException();
		}
		else if (entries.length == 0) {
			throw new IllegalArgumentException();
		}

		targetEntries = entries;
		for (int nIndex = 0; nIndex < targetEntries.length; nIndex++) {
			if (targetEntries[nIndex].exists() == false) {
				throw new IllegalStateException();
			}
			if (targetEntries[nIndex].isDirectory()) {
				throw new IllegalArgumentException();
			}
			else if (ZipUtilities.getFileType(targetEntries[nIndex]) == ZipUtilities.FileType.GZIP) {
				throw new IllegalArgumentException();
			}
		}
		gzipedEntries = new File[targetEntries.length];
	}

	//
	// 再定義メソッド(Command)
	//

	// 実行可能かを返す
	public boolean canExecute()
	{
		boolean canExecute = true;
		if (isExecuted() == false) {
			for (int nIndex = 0; nIndex < targetEntries.length; nIndex++) {
				if (targetEntries[nIndex].exists() == false
				|| targetEntries[nIndex].isDirectory()
				|| ZipUtilities.getFileType(targetEntries[nIndex]) == ZipUtilities.FileType.GZIP) {
					canExecute = false;
					break;
				}
			}
		}
		else {
			for (int nIndex = 0; nIndex < gzipedEntries.length; nIndex++) {
				if (gzipedEntries[nIndex] == null
				|| gzipedEntries[nIndex].exists() == false
				|| gzipedEntries[nIndex].isDirectory()
				|| ZipUtilities.getFileType(gzipedEntries[nIndex]) != ZipUtilities.FileType.GZIP) {
					canExecute = false;
					break;
				}
			}
		}
		
		return canExecute;
	}

	//
	// 再定義メソッド(ProcessEachEntryCommand)
	//

	// 処理エントリ数を返す
	protected int getProcessEntryCount()
	{
		return targetEntries.length;
	}
	
	// 処理名を返す
	protected String getOperationName()
	{
		return (this.isExecuted() == false ? "GZip file(s)" : "Cancel(delete) GZiped file(s)");
	}
	
	// 実行する	
	protected void process(
		YamlSerializer logSerializer,	// ログ書き込み
		CountReporter reporter)			// レポーター
		throws IOException
	{
		if (logSerializer == null) {
			throw new NullPointerException();
		}
		if (reporter == null) {
			throw new NullPointerException();
		}

		for (int nIndex = 0; nIndex < targetEntries.length; nIndex++) {
			// GZIP圧縮ファイル名を作成する
			// この時点でファイル名をフィールドに保存する
			gzipedEntries[nIndex] = makeGZipEntry(targetEntries[nIndex]);

			InputStream from = null;				
			OutputStream to = null;				
			try {
				from = new BufferedInputStream(new FileInputStream(targetEntries[nIndex]));
				to = new GZIPOutputStream(new BufferedOutputStream(new FileOutputStream(gzipedEntries[nIndex])));
				FileUtilities.duplicateStream(from, to);
			}
			finally {
				if (to != null) {
					to.close();
				}
				if (from != null) {
					from.close();
				}
			}
				
			System.err.println(targetEntries[nIndex].getName()
								+ " -> "
								+ gzipedEntries[nIndex].getName());
			logSerializer.writeMapping(targetEntries[nIndex].getName(), gzipedEntries[nIndex].getName());
			reporter.reportCountUp();
		}
	}
	
	// 処理をキャンセルする
	protected void cancelProcess(
		YamlSerializer logSerializer,	// ログ書き込み
		CountReporter reporter)			// レポーター
		throws IOException
	{
		if (logSerializer == null) {
			throw new NullPointerException();
		}

		// GZIP圧縮したファイルを削除する
		for (int nIndex = 0; nIndex < gzipedEntries.length; nIndex++) {
			// 圧縮失敗時も処理するため参照をチェックする
			if (gzipedEntries[nIndex] != null) {
				if (gzipedEntries[nIndex].exists()) {
					gzipedEntries[nIndex].delete();
				}
				
				System.err.println("Delete: " + gzipedEntries[nIndex].getName());
				logSerializer.writeSequence(gzipedEntries[nIndex].getName());
				if (reporter != null) {
					reporter.reportCountUp();
				}
				
				gzipedEntries[nIndex] = null;
			}
		}
	}
}
