/*
 * JavaFileUtilities: 
 */

package org.dbc_works.lib.io;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.jar.JarInputStream;
import java.util.jar.Manifest;

// 注記: インスタンスは作成させない
public final class JavaFileUtilities
{
	//
	// 内部型定義
	//

	// FileType: Javaから見たファイル種別
	// 『Effective Java』「項目21 enum構文をクラスで置き換える」に従って実装
	static public final class FileType
	{
		//
		// クラス属性
		//

		public static final FileType NOTJAVA	= new FileType("NotJava");
		public static final FileType SORUCE	= new FileType("Source");
		public static final FileType CLASS	= new FileType("Class");
		public static final FileType JAR	= new FileType("Jar");

		//
		// 属性
		//
		private final String name;

		//
		// コンストラクタ
		//
		private FileType(
			String strName)	// 名前
		{
			 name = strName;
		}

		//
		// 再定義メソッド(Object)
		//

		// 文字列化した結果を返す
		public String toString()
		{
			return name;
		}
	}

	//
	// クラスメソッド
	//

	// ルートを示すFileクラスのインスタンスを返す
	public static File getRootFile(
		File file)	// ルートを取得するファイル
	{
		File fileRoot = file.getAbsoluteFile();
		File fileParent;
		while ((fileParent = fileRoot.getParentFile()) != null) {
			fileRoot = fileParent;
		}
		return fileRoot;
	}

	// Javaファイルとしての種別を返す
	public static FileType getFileType(
		File file)	// 判定ファイルのパス
	{
		if (file == null) {
			throw new NullPointerException();
		}
		else if (file.exists() == false || file.isFile() == false) {
			throw new IllegalArgumentException();
		}

		FileType type = FileType.NOTJAVA;
		String strCompareFileName = file.getName().toLowerCase();
		if (strCompareFileName.endsWith(".java")) {
			type = FileType.SORUCE;
		}
		else if (strCompareFileName.endsWith(".class")) {
			type = FileType.CLASS;
		}
		else if (strCompareFileName.endsWith(".jar")) {
			type = FileType.JAR;
		}

		return type;
	}

	// Java実行ファイルかを返す
	public static boolean isExecutableJavaFile(
		File file)	// 判定ファイル
	{
		if (file == null) {
			throw new NullPointerException();
		}
		else if (file.exists() == false || file.isFile() == false) {
			throw new IllegalArgumentException();
		}

		boolean bCanExecute;
		final FileType type = getFileType(file);
		if (type == FileType.JAR) {
			try {
				bCanExecute = (getJarStartClass(file) != null); 
			}
			catch (IOException exp) {
				bCanExecute = false; 
			}
		}
		else {
			bCanExecute = (type == FileType.CLASS);
		}

		return bCanExecute;
	}
	
	// jarファイルから開始クラス名を取りだして返す
	public static String getJarStartClass(
		File jarFile)	// 対象jarファイル
		throws IOException
	{
		if (jarFile == null) {
			throw new NullPointerException();
		}
		else if (jarFile.exists() == false
		|| jarFile.isFile() == false
		|| getFileType(jarFile) != FileType.JAR) {
			throw new IllegalArgumentException();
		}

		String strJarStartClass = null;
		JarInputStream jis = null;
		try {
			jis = new JarInputStream(new BufferedInputStream(new FileInputStream(jarFile)));
			final Manifest manifest = jis.getManifest();
			if (manifest != null) {
				strJarStartClass = manifest.getMainAttributes().getValue("Main-Class");
			}
		}
		finally {
			if (jis != null) {
				try {
					jis.close();
				}
				catch (Exception e){
				}
			}
		}

		return strJarStartClass;
	}

	//
	// コンストラクタ
	//
	private JavaFileUtilities()
	{
	}
}
