/*
 * Document:
 * 注記: 実装は検討中のもの 
 * ObserverパターンのConcreteSubjectクラス 
 */

package org.dbc_works.lib.model;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Observable;

abstract public class Document extends Observable
{
	//
	// 定数
	//

	// バックアップファイルの拡張子
	static final String EXT_BACKUP	= ".bak";

	//
	// 属性
	//

	// 元となるリソース
	private Object sourceResource;

	// コマンドリスト
	private final CommandList commandList	= new CommandList();

	//
	// コンストラクタ
	//
	protected Document()
	{
		super();
	}

	//
	// 状態
	//

	// 取り消し可能かを返す
	final public boolean canUndo()
	{
		return commandList.canUndo();
	}

	// 取り消し可能コマンドの名前を返す
	final public String getUndoCommandName()
	{
		return commandList.getUndoCommandName();
	}

	// 再実行可能かを返す
	final public boolean canRedo()
	{
		return commandList.canRedo();
	}

	// 再実行可能コマンドの名前を返す
	final public String getRedoCommandName()
	{
		return commandList.getRedoCommandName();
	}

	// 修正されたかを返す
	final public boolean isModified()
	{
		return commandList.isModified();
	}
	
	// 保存可能かを返す
	final public boolean canWrite()
	{
		boolean bCanWrite = false;
		if (isSourceFile()) {
			bCanWrite = ((File)sourceResource).canWrite();
		}
		return bCanWrite;
	}

	// 結びつけられているFileのインスタンスのコピーを返す 
	final public File getFile()
	{
		if (isSourceFile() == false) {
			throw new IllegalStateException();
		}
		return new File(((File)sourceResource).getAbsolutePath());
	}

	//
	// 操作
	//

	// 引数が示すリソースと結びつける
	final public void bindResource(
		File file)	// 元となるファイル
		throws IOException
	{
		if (file == null) {
			throw new NullPointerException();
		}
		else if (file.exists() == false
		|| file.isFile() == false
		|| file.canRead() == false) {
			throw new IllegalArgumentException();
		}

		if (sourceResource != null) {
			throw new IllegalStateException();
		}

		read(file.toURL());
		sourceResource = file;
	}
	final public void bindResource(
		URL url)	// 元となるリソースのURL
		throws IOException
	{
		if (url == null) {
			throw new NullPointerException();
		}

		if (sourceResource != null) {
			throw new IllegalStateException();
		}

		read(url);
		sourceResource = url;
	}
	
	// 与えられたコマンドを実行する
	final public void execute(
		Command command)	// 実行コマンド
		throws Exception
	{
		if (command == null) {
			throw new NullPointerException();
		}

		commandList.execute(command);
	}

	// 直前に与えられたコマンドの実行を取り消す
	final public void undo()
		throws Exception
	{
		if (canUndo() == false) {
			throw new IllegalStateException();
		}
		commandList.undo();
	}
	
	// 取り消した直前のコマンドを再実行する
	final public void redo()
	throws Exception
	{
		if (canRedo() == false) {
			throw new IllegalStateException();
		}
		commandList.redo();
	}

	// 書きこむ
	final public void write()
		throws IOException
	{
		if (canWrite() == false) {
			throw new IllegalStateException();
		}

		File fileBackup = makeBackupFile();
		if (fileBackup.exists()) {
			fileBackup.delete();
		}

		File fileSource = (File)sourceResource;
		fileSource.renameTo(fileBackup);

		try {
			write(fileSource);
		}
		catch (IOException e) {
			fileSource.delete();
			fileBackup.renameTo(fileSource);
			throw e;
		}
	}
	final public void writeAs(
		File fileWrite)	// 書きこみ対象ファイル
		throws IOException
	{
		if (fileWrite == null) {
			throw new NullPointerException();
		}
		else if (fileWrite.exists() != false && fileWrite.canWrite() == false) {
			throw new IllegalArgumentException();
		}

		write(fileWrite);

		sourceResource = fileWrite;
	}

	//
	// 実装
	//

	// ソースがファイルかを返す
	private boolean isSourceFile()
	{
		return sourceResource instanceof File;
	}

	// バックアップファイルをあらわすFileインスタンスを返す
	private File makeBackupFile()
	{
		if (isSourceFile() == false) {
			throw new IllegalStateException();
		}
		return new File(((File)sourceResource).getAbsoluteFile() + EXT_BACKUP);
	}

	// URLから読みこむ
	private void read(
		URL urlSource)	// 元となるリソースのURL
		throws IOException
	{
		if (urlSource == null) {
			throw new NullPointerException();
		}

		InputStream inputStream = null; 
		try {
			inputStream = urlSource.openStream(); 
			readFrom(inputStream);
		}
		finally {
			if (inputStream != null) {
				inputStream.close();
			}
		}
	}

	// ファイルに書きこむ
	private void write(
		File file)	// 書きこみ対象ファイル 
		throws IOException
	{
		if (file == null) {
			throw new NullPointerException();
		}

		OutputStream stream = null;
		try {
			stream = new FileOutputStream(file);
			writeTo(stream);
		}
		finally {
			if (stream != null) {
				stream.close();
			}
		}
	}

	// ファイル名を返す
	final protected String getFileName()
	{
		String strFileName = null;
		if (sourceResource instanceof File) {
			strFileName = ((File)sourceResource).getName();
		}
		else if (sourceResource instanceof URL) {
			// これだとパスやクエリ文字列など余計なものも返されるかもしれない
			strFileName = ((URL)sourceResource).getFile();
		}
		else if (sourceResource != null) {
			throw new IllegalStateException();
		}
		return strFileName;
	}

	//
	// 再定義必須メソッド
	//

	// ストリームから読みこむ
	protected abstract void readFrom(InputStream stream) throws IOException;

	// ストリームに書きこむ
	protected abstract void writeTo(OutputStream stream) throws IOException;
}
